% SAMM 2017, exercise 2: optimal control of a pendulum on a cart with forcing
% by a Variational Integrator discretization (linear interpolation, m.p.-rule)
% 
% author: kathrin.flasskamp@uni-bremen.de
%
% [] = visualization(t,q,u)
%
% generates a "movie" of cart and pendulum trajectory with force visualized by 
% yellow bar
%
% Input:  t vector of time points
%         q matrix of configurations ((N+1)x2), q=[cart pos x, pendulum angle a]
%         u vector of controls (Nx1)
% Output: []


function [] = visualization(t,q,u)

  figure;
  hold on;
  axis([-1 3 -2 2])
  axis square;
  
  [sx,sy,sz] = sphere(20);

  u = [u;0]; % append 0 for simplicity, to have the same length as q and t
  % although u(i) is meant to be applied during a time-interval not at nodes
  
  rest_time = 10/length(t); %time between frames is adjusted to coarseness of
  % discretization. Could use an interpolation instead for smoothness in movie.
  
  for i=1:1:length(t)
    clf
        
    axis([-1 3 -2 2])
    
    
    % plot cart: use q_i for lower left position of rectangle
    rectangle("Position",[q(i,1),0,1,.5],"FaceColor",'b')
    hold on
    surf (0.1*sx+q(i,1)+0.25,0.1*sy,0.1*sz);
    surf (0.1*sx+q(i,1)+0.75,0.1*sy,0.1*sz);   
    hold on
    
    % plot pendulum: define joint point on cart and pivot point at mass
    joint = [q(i,1)+.5;.5];
    pivot = joint + [cos(q(i,2));sin(q(i,2))];
    line([joint(1),pivot(1)],[joint(2),pivot(2)],"Color",'r',"LineWidth",8.0);
    surf (0.15*sx+pivot(1),0.15*sy+pivot(2),0.15*sz);
    
    % plot control force as bar on cart, force scaled by factor 30
    line([q(i,1)+.5 q(i,1)+.5+u(i)/30],[0.25 0.25],"Color",'y',"LineWidth",6.0);
    
       
    axis([-1 3 -2 2])
    axis square
    pause(rest_time)
  endfor


endfunction